#!/usr/bin/env python

import sys
import tempfile
import gzip
import bz2
import os

args = sys.argv[1:]
if len(args)==2:
    input_filename, output_filename = args
elif len(args)==1:
    input_filename, output_filename = args[0], None
else:
    print "Usage: python sortosc.py input_filename [output_filename]"
 

if input_filename.endswith(".gz"):
    input = gzip.open(input_filename)
elif input_filename.endswith(".bz2"):
    input = bz2.BZ2File(input_filename)
else:
    input = open(input_filename)

if output_filename:
    if output_filename.endswith(".gz"):
        output = gzip.open(output_filename, 'w')
    elif input_filename.endswith(".bz2"):
        output = bz2.BZ2File(output_filename, 'w')
    else:
        output = open(output_filename, 'w')
else:
    output = tempfile.NamedTemporaryFile('w', delete=False)

for line in input:
    if line.startswith("##ChromosomeNameOrder"):
        key, value = line.split("=")
        assert key.strip()=="##ChromosomeNameOrder"
        chromosomes = value.strip().split(",")
    output.write(line)
    if not line.startswith("#"):
        break
 
words = line.split()
chromosome_index = words.index("chrom")
if 'start.0base' in words:
    start_index = words.index("start.0base")
elif 'start.1base' in words:
    start_index = words.index("start.1base")
else:
    raise Exception("No starting position found in file")
end_index = words.index("end")
strand_index = words.index("strand")

lines = {}
for chromosome in chromosomes:
    lines[chromosome] = {}

print "Reading input"

for line in input:
    words = line.split()
    chromosome = words[chromosome_index]
    start = int(words[start_index])
    end = int(words[end_index])
    strand = words[strand_index]
    position =  (start, end, strand)
    lines[chromosome][position] = line

input.close()

for chromosome in chromosomes:
    print "Writing %s to %s" % (chromosome, output.name)
    positions = lines[chromosome].keys()
    positions.sort()
    for position in positions:
        line = lines[chromosome][position]
        output.write(line)

output.close()

if not output_filename:
    output_filename = output.name
    if input_filename.endswith(".gz"):
        command = 'gzip -c %s > %s' % (output_filename, input_filename)
        print command
        os.system(command)
        print "Removing %s" % output_filename
        os.remove(output_filename)
    elif input_filename.endswith(".bz2"):
        command = 'bzip2 -c %s > %s' % (output_filename, input_filename)
        print command
        os.system(command)
        print "Removing %s" % output_filename
        os.remove(output_filename)
    else:
        print "Renaming %s to %s" % (output_filename, input_filename)
        os.rename(output_filename, input_filename)
